/*
----------------------------------------
What is Key function ???
----------------------------------------
	-> When D3 performs a data join it joins the first array 
  element to the first element in the selection, 
  the second array element to the second element in the 
  selection and so on. However, if the order of array elements changes 
  (due to sorting, inserting or removing elements) the array elements 
  can get joined to different DOM elements.
  
  -> You can ensure each array element remains joined to the same 
  HTML/SVG element by passing a key function into the .data method. 
  
  -> The key function should return a unique id value for each 
  array element.

*/


let letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
let i = 25;

function doInsert() {
  if(i < 0)
    return;
  
  let myData = letters.slice(i).split('');
  i--;
  update(myData); // Update Without key function
  updateWithKey(myData); // Update with key function
};

function update(data) {
  d3
    .select('#content')
  	.selectAll('div')
  	.data(data)
  	.join('div')
  	.transition()
  	.style('left', (d, i) => i * 32 + 'px')
  	.text(d => d)
}

function updateWithKey(data) {
  d3
    .select('#content2')
  	.selectAll('div')
  	.data(data, d => d) // Key Function
  	.join('div')
  	.transition()
  	.style('left', (d, i) => i * 32 + 'px')
  	.text(d => d)
}

doInsert();